from engine_factory import CdmEngineFactory
from cdm6_tables import metadata
from sqlalchemy.sql import select
import datetime

cdm = CdmEngineFactory()

engine = cdm.engine
metadata.create_all(engine)
metadata.reflect(bind=engine)
for table in reversed(metadata.sorted_tables):
    print(table)



Cohort = cdm.base.cohort
session =  cdm.session
session.add(Cohort(cohort_definition_id=2, subject_id=100, 
            cohort_end_date=datetime.datetime.now(), 
            cohort_start_date=datetime.datetime.now()))
session.commit()

s = select([Cohort])
result = session.execute(s)
for row in result:
    print(row)


# -*- coding: utf-8 -*-
"""
    Dummy conftest.py for pyomop.

    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    https://pytest.org/latest/plugins.html
"""

import asyncio
import pytest

@pytest.fixture
def pyomop_fixture():
    from src.pyomop import CdmEngineFactory
    cdm = CdmEngineFactory()
    return cdm

@pytest.fixture
def metadata_fixture():
    from src.pyomop import metadata
    return metadata


@pytest.fixture
def vector_fixture():
    from src.pyomop import CdmVector
    return CdmVector()


# https://github.com/OHDSI/QueryLibrary/tree/master/inst/shinyApps/QueryLibrary/queries

# Test
TEST = "SELECT * from cohort;"


# PE03: Number of patients grouped by gender
PE03 = """
SELECT
  person.gender_concept_id,
  concept.concept_name    AS gender_name,
  COUNT(person.person_id) AS num_persons
FROM person
  JOIN concept ON person.gender_concept_id = concept.concept_id
GROUP BY person.gender_concept_id, concept.concept_name;
"""
CDMSQL = {
    "TEST": TEST,
    "PE03": PE03
}




def test_vocab(pyomop_fixture, capsys):
    from src.pyomop import CdmVocabulary
    vocab = CdmVocabulary(pyomop_fixture)
    # for x in pyomop_fixture.base:
    #     print(x)
    # setters are called like this
    vocab.concept_id = 45956935 # 'C0020538' Hypertension
    assert(vocab.concept_name == 'xxxx xxxxxxx')





import click
import asyncio
from . import CdmEngineFactory
from . import CdmVocabulary
from . import metadata
from . import __version__


@click.command()
@click.option('--verbose', '-v', is_flag=True, help="Will print verbose messages.")
@click.option('--create', '-c', is_flag=True, help="Create CDM6 schema on the database.")
@click.option('--dbtype', '-t', multiple=False, default='sqlite',
              help='Database Type for creating CDM (sqlite, mysql or pgsql')
@click.option('--host', '-h', multiple=False, default='localhost',
              help='Database host')
@click.option('--port', '-p', multiple=False, default='5432',
              help='Database port')
@click.option('--user', '-u', multiple=False, default='root',
              help='Database user')
@click.option('--pw', '-w', multiple=False, default='pass',
              help='Database password')
@click.option('--name', '-n', multiple=False, default='cdm6.sqlite',
              help='Database name')
@click.option('--schema', '-s', multiple=False, default='public',
              help='Database schema (for pgsql)')
@click.option('--vocab', '-i', multiple=False, default='',
              help='Folder with vocabulary files (csv) to import')
def cli(verbose, create, dbtype, host, port, user, pw, name, schema, vocab):
    if verbose:
        print("verbose")
    if create:
        cdm = CdmEngineFactory(dbtype, host, port, user, pw, name, schema)
        # initialize default engine
        engine = cdm.engine
        asyncio.run(cdm.init_models(metadata))
    if vocab is not '':
        cdm = CdmEngineFactory(dbtype, host, port, user, pw, name, schema)
        _vocab = CdmVocabulary(cdm)
        _vocab.create_vocab(vocab)
        print("Done")

def main_routine():
    click.echo("_________________________________________")
    click.echo("Pyomop v" + __version__ + " working:.....")
    cli()  # run the main function
    click.echo("Pyomop done.")


if __name__ == '__main__':
    main_routine()


"""
 Copyright (C) 2020 Bell Eapen

 This file is part of PyOMOP.

 PyOMOP is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 PyOMOP is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with PyOMOP.  If not, see <http://www.gnu.org/licenses/>.
"""

# coding: utf-8
from sqlalchemy import BigInteger, Column, Integer, Numeric, String, Text
from sqlalchemy.orm import declarative_base

Base = declarative_base()
metadata = Base.metadata


class AttributeDefinition(Base):
    __tablename__ = 'attribute_definition'

    # _id = Column(Integer, primary_key=True)
    attribute_definition_id = Column(Integer, primary_key=True)
    attribute_name = Column(String(255), nullable=False)
    attribute_description = Column(Text)
    attribute_type_concept_id = Column(Integer, nullable=False)
    attribute_syntax = Column(Text)


class CareSite(Base):
    __tablename__ = 'care_site'

    # _id = Column(Integer, primary_key=True)
    care_site_id = Column(Integer, primary_key=True)
    care_site_name = Column(String(255))
    place_of_service_concept_id = Column(Integer)
    location_id = Column(Integer)
    care_site_source_value = Column(String(50))
    place_of_service_source_value = Column(String(50))


class CdmSource(Base):
    __tablename__ = 'cdm_source'

    # _id = Column(Integer, primary_key=True)
    cdm_source_name = Column(String(255), primary_key=True)
    cdm_source_abbreviation = Column(String(25))
    cdm_holder = Column(String(255))
    source_description = Column(Text)
    source_documentation_reference = Column(String(255))
    cdm_etl_reference = Column(String(255))
    source_release_date = Column(String(30))
    cdm_release_date = Column(String(30))
    cdm_version = Column(String(10))
    vocabulary_version = Column(String(20))


class Cohort(Base):
    __tablename__ = 'cohort'

    _id = Column(Integer, primary_key=True)
    cohort_definition_id = Column(Integer, nullable=False)
    subject_id = Column(Integer, nullable=False)
    cohort_start_date = Column(String(30), nullable=False)
    cohort_end_date = Column(String(30), nullable=False)


class CohortAttribute(Base):
    __tablename__ = 'cohort_attribute'

    _id = Column(Integer, primary_key=True)
    cohort_definition_id = Column(Integer, nullable=False)
    cohort_start_date = Column(String(30), nullable=False)
    cohort_end_date = Column(String(30), nullable=False)
    subject_id = Column(Integer, nullable=False)
    attribute_definition_id = Column(Integer, nullable=False)
    value_as_number = Column(Numeric)
    value_as_concept_id = Column(Integer)


class CohortDefinition(Base):
    __tablename__ = 'cohort_definition'

    # _id = Column(Integer, primary_key=True)
    cohort_definition_id = Column(Integer, primary_key=True)
    cohort_definition_name = Column(String(255), nullable=False)
    cohort_definition_description = Column(Text)
    definition_type_concept_id = Column(Integer, nullable=False)
    cohort_definition_syntax = Column(Text)
    subject_concept_id = Column(Integer, nullable=False)
    cohort_initiation_date = Column(String(30))


class Concept(Base):
    __tablename__ = 'concept'

    # _id = Column(Integer, primary_key=True)
    concept_id = Column(Integer, primary_key=True)
    concept_name = Column(String(255), nullable=False)
    domain_id = Column(String(20), nullable=False)
    vocabulary_id = Column(String(20), nullable=False)
    concept_class_id = Column(String(20), nullable=False)
    standard_concept = Column(String(1))
    concept_code = Column(String(50), nullable=False)
    valid_start_date = Column(String(30), nullable=False)
    valid_end_date = Column(String(30), nullable=False)
    invalid_reason = Column(String(1))


class ConceptAncestor(Base):
    __tablename__ = 'concept_ancestor'

    _id = Column(Integer, primary_key=True)
    ancestor_concept_id = Column(Integer, nullable=False)
    descendant_concept_id = Column(Integer, nullable=False)
    min_levels_of_separation = Column(Integer, nullable=False)
    max_levels_of_separation = Column(Integer, nullable=False)


class ConceptClass(Base):
    __tablename__ = 'concept_class'

    # _id = Column(Integer, primary_key=True)
    concept_class_id = Column(String(20), primary_key=True)
    concept_class_name = Column(String(255), nullable=False)
    concept_class_concept_id = Column(Integer, nullable=False)


class ConceptRelationship(Base):
    __tablename__ = 'concept_relationship'

    _id = Column(Integer, primary_key=True)
    concept_id_1 = Column(Integer, nullable=False)
    concept_id_2 = Column(Integer, nullable=False)
    relationship_id = Column(String(20), nullable=False)
    valid_start_date = Column(String(30), nullable=False)
    valid_end_date = Column(String(30), nullable=False)
    invalid_reason = Column(String(1))


class ConceptSynonym(Base):
    __tablename__ = 'concept_synonym'

    _id = Column(Integer, primary_key=True)
    concept_id = Column(Integer, nullable=False)
    concept_synonym_name = Column(String(1000), nullable=False)
    language_concept_id = Column(Integer, nullable=False)


class ConditionEra(Base):
    __tablename__ = 'condition_era'

    # _id = Column(Integer, primary_key=True)
    condition_era_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    condition_concept_id = Column(Integer, nullable=False)
    condition_era_start_date = Column(String(30), nullable=False)
    condition_era_end_date = Column(String(30), nullable=False)
    condition_occurrence_count = Column(Integer)


class ConditionOccurrence(Base):
    __tablename__ = 'condition_occurrence'

    # _id = Column(Integer, primary_key=True)
    condition_occurrence_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    condition_concept_id = Column(Integer, nullable=False)
    condition_start_date = Column(String(30), nullable=False)
    condition_end_date = Column(String(30))
    condition_type_concept_id = Column(Integer, nullable=False)
    stop_reason = Column(String(20))
    provider_id = Column(Integer)
    visit_occurrence_id = Column(BigInteger)
    condition_source_value = Column(String(50))
    condition_source_concept_id = Column(Integer)


class Death(Base):
    __tablename__ = 'death'

    _id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    death_date = Column(String(30), nullable=False)
    death_type_concept_id = Column(Integer, nullable=False)
    cause_concept_id = Column(Integer)
    cause_source_value = Column(String(50))
    cause_source_concept_id = Column(Integer)


class DeviceCost(Base):
    __tablename__ = 'device_cost'

    # _id = Column(Integer, primary_key=True)
    device_cost_id = Column(Integer, primary_key=True)
    device_exposure_id = Column(Integer, nullable=False)
    currency_concept_id = Column(Integer)
    paid_copay = Column(Numeric)
    paid_coinsurance = Column(Numeric)
    paid_toward_deductible = Column(Numeric)
    paid_by_payer = Column(Numeric)
    paid_by_coordination_benefits = Column(Numeric)
    total_out_of_pocket = Column(Numeric)
    total_paid = Column(Numeric)
    payer_plan_period_id = Column(Integer)


class DeviceExposure(Base):
    __tablename__ = 'device_exposure'

    # _id = Column(Integer, primary_key=True)
    device_exposure_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    device_concept_id = Column(Integer, nullable=False)
    device_exposure_start_date = Column(String(30), nullable=False)
    device_exposure_end_date = Column(String(30))
    device_type_concept_id = Column(Integer, nullable=False)
    unique_device_id = Column(String(50))
    quantity = Column(Integer)
    provider_id = Column(Integer)
    visit_occurrence_id = Column(BigInteger)
    device_source_value = Column(String(100))
    device_source_concept_id = Column(Integer)


class Domain(Base):
    __tablename__ = 'domain'

    # _id = Column(Integer, primary_key=True, nullable=False)
    domain_id = Column(String(20), primary_key=True)
    domain_name = Column(String(255), nullable=False)
    domain_concept_id = Column(Integer, nullable=False)


class DoseEra(Base):
    __tablename__ = 'dose_era'

    # _id = Column(Integer, primary_key=True)
    dose_era_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    drug_concept_id = Column(Integer, nullable=False)
    unit_concept_id = Column(Integer, nullable=False)
    dose_value = Column(Numeric, nullable=False)
    dose_era_start_date = Column(String(30), nullable=False)
    dose_era_end_date = Column(String(30), nullable=False)


class DrugCost(Base):
    __tablename__ = 'drug_cost'

    # _id = Column(Integer, primary_key=True)
    drug_cost_id = Column(Integer, primary_key=True)
    drug_exposure_id = Column(Integer, nullable=False)
    currency_concept_id = Column(Integer)
    paid_copay = Column(Numeric)
    paid_coinsurance = Column(Numeric)
    paid_toward_deductible = Column(Numeric)
    paid_by_payer = Column(Numeric)
    paid_by_coordination_benefits = Column(Numeric)
    total_out_of_pocket = Column(Numeric)
    total_paid = Column(Numeric)
    ingredient_cost = Column(Numeric)
    dispensing_fee = Column(Numeric)
    average_wholesale_price = Column(Numeric)
    payer_plan_period_id = Column(Integer)


class DrugEra(Base):
    __tablename__ = 'drug_era'

    # _id = Column(Integer, primary_key=True)
    drug_era_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    drug_concept_id = Column(Integer, nullable=False)
    drug_era_start_date = Column(String(30), nullable=False)
    drug_era_end_date = Column(String(30), nullable=False)
    drug_exposure_count = Column(Integer)
    gap_days = Column(Integer)


class DrugExposure(Base):
    __tablename__ = 'drug_exposure'

    # _id = Column(Integer, primary_key=True)
    drug_exposure_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    drug_concept_id = Column(Integer, nullable=False)
    drug_exposure_start_date = Column(String(30), nullable=False)
    drug_exposure_end_date = Column(String(30))
    drug_type_concept_id = Column(Integer, nullable=False)
    stop_reason = Column(String(20))
    refills = Column(Integer)
    quantity = Column(Numeric)
    days_supply = Column(Integer)
    sig = Column(Text)
    route_concept_id = Column(Integer)
    effective_drug_dose = Column(Numeric)
    dose_unit_concept_id = Column(Integer)
    lot_number = Column(String(50))
    provider_id = Column(Integer)
    visit_occurrence_id = Column(BigInteger)
    drug_source_value = Column(String(50))
    drug_source_concept_id = Column(Integer)
    route_source_value = Column(String(50))
    dose_unit_source_value = Column(String(50))


class DrugStrength(Base):
    __tablename__ = 'drug_strength'

    # TODO Check this
    # _id = Column(Integer, primary_key=True)
    # drug_concept_id = Column(Integer, nullable=False)
    drug_concept_id = Column(Integer, primary_key=True)
    ingredient_concept_id = Column(Integer, nullable=False)
    amount_value = Column(Numeric)
    amount_unit_concept_id = Column(Integer)
    numerator_value = Column(Numeric)
    numerator_unit_concept_id = Column(Integer)
    denominator_value = Column(Numeric)
    denominator_unit_concept_id = Column(Integer)
    box_size = Column(Integer)
    valid_start_date = Column(String(30), nullable=False)
    valid_end_date = Column(String(30), nullable=False)
    invalid_reason = Column(String(1))


class FactRelationship(Base):
    __tablename__ = 'fact_relationship'

    _id = Column(Integer, primary_key=True)
    domain_concept_id_1 = Column(Integer, nullable=False)
    fact_id_1 = Column(Integer, nullable=False)
    domain_concept_id_2 = Column(Integer, nullable=False)
    fact_id_2 = Column(Integer, nullable=False)
    relationship_concept_id = Column(Integer, nullable=False)


class Location(Base):
    __tablename__ = 'location'

    # _id = Column(Integer, primary_key=True)
    location_id = Column(Integer, primary_key=True)
    address_1 = Column(String(50))
    address_2 = Column(String(50))
    city = Column(String(50))
    state = Column(String(2))
    zip = Column(String(9))
    county = Column(String(20))
    location_source_value = Column(String(50))


class Measurement(Base):
    __tablename__ = 'measurement'

    # _id = Column(Integer, primary_key=True)
    measurement_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    measurement_concept_id = Column(Integer, nullable=False)
    measurement_date = Column(String(30), nullable=False)
    measurement_time = Column(String(10))
    measurement_type_concept_id = Column(Integer, nullable=False)
    operator_concept_id = Column(Integer)
    value_as_number = Column(Numeric)
    value_as_concept_id = Column(Integer)
    unit_concept_id = Column(Integer)
    range_low = Column(Numeric)
    range_high = Column(Numeric)
    provider_id = Column(Integer)
    visit_occurrence_id = Column(BigInteger)
    measurement_source_value = Column(String(50))
    measurement_source_concept_id = Column(Integer)
    unit_source_value = Column(String(50))
    value_source_value = Column(String(50))


class Note(Base):
    __tablename__ = 'note'

    # _id = Column(Integer, primary_key=True)
    note_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    note_date = Column(String(30), nullable=False)
    note_time = Column(String(10))
    note_type_concept_id = Column(Integer, nullable=False)
    note_text = Column(Text, nullable=False)
    provider_id = Column(Integer)
    visit_occurrence_id = Column(BigInteger)
    note_source_value = Column(String(50))


class Observation(Base):
    __tablename__ = 'observation'

    # _id = Column(Integer, primary_key=True)
    observation_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    observation_concept_id = Column(Integer, nullable=False)
    observation_date = Column(String(30), nullable=False)
    observation_time = Column(String(10))
    observation_type_concept_id = Column(Integer, nullable=False)
    value_as_number = Column(Numeric)
    value_as_string = Column(String(60))
    value_as_concept_id = Column(Integer)
    qualifier_concept_id = Column(Integer)
    unit_concept_id = Column(Integer)
    provider_id = Column(Integer)
    visit_occurrence_id = Column(BigInteger)
    observation_source_value = Column(String(50))
    observation_source_concept_id = Column(Integer)
    unit_source_value = Column(String(50))
    qualifier_source_value = Column(String(50))


class ObservationPeriod(Base):
    __tablename__ = 'observation_period'

    # _id = Column(Integer, primary_key=True)
    observation_period_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    observation_period_start_date = Column(String(30), nullable=False)
    observation_period_end_date = Column(String(30), nullable=False)
    period_type_concept_id = Column(Integer, nullable=False)


class PayerPlanPeriod(Base):
    __tablename__ = 'payer_plan_period'

    # _id = Column(Integer, primary_key=True)
    payer_plan_period_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    payer_plan_period_start_date = Column(String(30), nullable=False)
    payer_plan_period_end_date = Column(String(30), nullable=False)
    payer_source_value = Column(String(50))
    plan_source_value = Column(String(50))
    family_source_value = Column(String(50))


class Person(Base):
    __tablename__ = 'person'

    #_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, primary_key=True)
    gender_concept_id = Column(Integer, nullable=False)
    year_of_birth = Column(Integer, nullable=False)
    month_of_birth = Column(Integer)
    day_of_birth = Column(Integer)
    #time_of_birth = Column(String(10))
    birth_datetime = Column(String(10))
    death_datetime = Column(String(10))
    race_concept_id = Column(Integer, nullable=False)
    ethnicity_concept_id = Column(Integer, nullable=False)
    location_id = Column(Integer)
    provider_id = Column(Integer)
    care_site_id = Column(Integer)
    person_source_value = Column(String(50))
    gender_source_value = Column(String(50))
    gender_source_concept_id = Column(Integer)
    race_source_value = Column(String(50))
    race_source_concept_id = Column(Integer)
    ethnicity_source_value = Column(String(50))
    ethnicity_source_concept_id = Column(Integer)


class ProcedureCost(Base):
    __tablename__ = 'procedure_cost'

    # _id = Column(Integer, primary_key=True)
    procedure_cost_id = Column(Integer, primary_key=True)
    procedure_occurrence_id = Column(Integer, nullable=False)
    currency_concept_id = Column(Integer)
    paid_copay = Column(Numeric)
    paid_coinsurance = Column(Numeric)
    paid_toward_deductible = Column(Numeric)
    paid_by_payer = Column(Numeric)
    paid_by_coordination_benefits = Column(Numeric)
    total_out_of_pocket = Column(Numeric)
    total_paid = Column(Numeric)
    revenue_code_concept_id = Column(Integer)
    payer_plan_period_id = Column(Integer)
    revenue_code_source_value = Column(String(50))


class ProcedureOccurrence(Base):
    __tablename__ = 'procedure_occurrence'

    # _id = Column(Integer, primary_key=True)
    procedure_occurrence_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    procedure_concept_id = Column(Integer, nullable=False)
    procedure_date = Column(String(30), nullable=False)
    procedure_type_concept_id = Column(Integer, nullable=False)
    modifier_concept_id = Column(Integer)
    quantity = Column(Integer)
    provider_id = Column(Integer)
    visit_occurrence_id = Column(BigInteger)
    procedure_source_value = Column(String(50))
    procedure_source_concept_id = Column(Integer)
    qualifier_source_value = Column(String(50))


class Provider(Base):
    __tablename__ = 'provider'

    # _id = Column(Integer, primary_key=True)
    provider_id = Column(Integer, primary_key=True)
    provider_name = Column(String(255))
    npi = Column(String(20))
    dea = Column(String(20))
    specialty_concept_id = Column(Integer)
    care_site_id = Column(Integer)
    year_of_birth = Column(Integer)
    gender_concept_id = Column(Integer)
    provider_source_value = Column(String(50))
    specialty_source_value = Column(String(50))
    specialty_source_concept_id = Column(Integer)
    gender_source_value = Column(String(50))
    gender_source_concept_id = Column(Integer)


class Relationship(Base):
    __tablename__ = 'relationship'

    # _id = Column(Integer, primary_key=True)
    relationship_id = Column(String(20), primary_key=True)
    relationship_name = Column(String(255), nullable=False)
    is_hierarchical = Column(String(1), nullable=False)
    defines_ancestry = Column(String(1), nullable=False)
    reverse_relationship_id = Column(String(20), nullable=False)
    relationship_concept_id = Column(Integer, nullable=False)


class SourceToConceptMap(Base):
    __tablename__ = 'source_to_concept_map'

    _id = Column(Integer, primary_key=True)
    source_code = Column(String(50), nullable=False)
    source_concept_id = Column(Integer, nullable=False)
    source_vocabulary_id = Column(String(20), nullable=False)
    source_code_description = Column(String(255))
    target_concept_id = Column(Integer, nullable=False)
    target_vocabulary_id = Column(String(20), nullable=False)
    valid_start_date = Column(String(30), nullable=False)
    valid_end_date = Column(String(30), nullable=False)
    invalid_reason = Column(String(1))


class Speciman(Base):
    __tablename__ = 'specimen'

    # _id = Column(Integer, primary_key=True)
    specimen_id = Column(Integer, primary_key=True)
    person_id = Column(Integer, nullable=False)
    specimen_concept_id = Column(Integer, nullable=False)
    specimen_type_concept_id = Column(Integer, nullable=False)
    specimen_date = Column(String(30), nullable=False)
    specimen_time = Column(String(10))
    quantity = Column(Numeric)
    unit_concept_id = Column(Integer)
    anatomic_site_concept_id = Column(Integer)
    disease_status_concept_id = Column(Integer)
    specimen_source_id = Column(String(50))
    specimen_source_value = Column(String(50))
    unit_source_value = Column(String(50))
    anatomic_site_source_value = Column(String(50))
    disease_status_source_value = Column(String(50))


class VisitCost(Base):
    __tablename__ = 'visit_cost'

    # _id = Column(Integer, primary_key=True)
    visit_cost_id = Column(Integer, primary_key=True)
    visit_occurrence_id = Column(BigInteger, nullable=False)
    currency_concept_id = Column(Integer)
    paid_copay = Column(Numeric)
    paid_coinsurance = Column(Numeric)
    paid_toward_deductible = Column(Numeric)
    paid_by_payer = Column(Numeric)
    paid_by_coordination_benefits = Column(Numeric)
    total_out_of_pocket = Column(Numeric)
    total_paid = Column(Numeric)
    payer_plan_period_id = Column(Integer)


class VisitOccurrence(Base):
    __tablename__ = 'visit_occurrence'

    # _id = Column(Integer, primary_key=True)
    visit_occurrence_id = Column(BigInteger, primary_key=True)
    person_id = Column(Integer, nullable=False)
    visit_concept_id = Column(Integer, nullable=False)
    visit_start_date = Column(String(30), nullable=False)
    visit_start_time = Column(String(10))
    visit_end_date = Column(String(30), nullable=False)
    visit_end_time = Column(String(10))
    visit_type_concept_id = Column(Integer, nullable=False)
    provider_id = Column(Integer)
    care_site_id = Column(Integer)
    visit_source_value = Column(String(50))
    visit_source_concept_id = Column(Integer)


class Vocabulary(Base):
    __tablename__ = 'vocabulary'

    # _id = Column(Integer, primary_key=True)
    vocabulary_id = Column(String(20), primary_key=True)
    vocabulary_name = Column(String(255), nullable=False)
    vocabulary_reference = Column(String(255), nullable=False)
    vocabulary_version = Column(String(255))
    vocabulary_concept_id = Column(Integer, nullable=False)



# from sqlalchemy import create_engine
# from sqlalchemy.orm import Session
# from sqlalchemy.ext.automap import automap_base

import asyncio
from sqlalchemy.ext.asyncio import create_async_engine
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.automap import automap_base

class CdmEngineFactory(object):

    def __init__(self, db = 'sqlite',
                host = 'localhost', port = 5432,
                user = 'root', pw='pass',
                name = 'cdm6.sqlite', schema = 'public'):
        self._db = db
        self._name = name
        self._host = host
        self._port = port
        self._user = user
        self._pw = pw
        self._schema = schema
        self._engine = None
        self._base = None

    async def init_models(self, metadata):
        async with self._engine.begin() as conn:
            await conn.run_sync(metadata.drop_all)
            await conn.run_sync(metadata.create_all)


    @property
    def db(self):
        return self._db

    @property
    def host(self):
        return self._host

    @property
    def port(self):
        return self._port

    @property
    def name(self):
        return self._name

    @property
    def user(self):
        return self._user

    @property
    def pw(self):
        return self._pw

    @property
    def schema(self):
        return self._schema

    @property
    def engine(self):
        return self.engine

    @property
    def base(self):
        if self.engine is not None:  # Not self_engine
            Base = automap_base()
            Base.prepare(self.engine, reflect=True)
            return Base.classes
        return None

    @property
    def engine(self):
        if self._db == 'sqlite':
            self._engine = create_async_engine("sqlite+aiosqlite:///"+self._name)
        if self._db == 'mysql':
            mysql_url = 'mysql://{}:{}@{}:{}/{}'
            mysql_url = mysql_url.format(self._user, self._pw, self._host, self._port, self._name)
            self._engine = create_async_engine(mysql_url, isolation_level="READ UNCOMMITTED")
        if self._db == 'pgsql':
            # https://stackoverflow.com/questions/9298296/sqlalchemy-support-of-postgres-schemas
            dbschema = '{},public'  # Searches left-to-right
            dbschema = dbschema.format(self._schema)
            pgsql_url = 'postgresql+psycopg2://{}:{}@{}:{}/{}'
            pgsql_url = pgsql_url.format(self._user, self._pw,
                                        self._host, self._port, self._name)
            self._engine = create_async_engine(
                pgsql_url,
                connect_args={'options': '-csearch_path={}'.format(dbschema)})
        return self._engine

    @property
    def session(self):
        if self._engine is not None:
            async_session = sessionmaker(self._engine, expire_on_commit=False, class_=AsyncSession)
            return async_session
        return None

    @property
    def async_session(self):
        if self._engine is not None:
            async_session = sessionmaker(self._engine, expire_on_commit=False, class_=AsyncSession)
            return async_session
        return None

    @db.setter
    def db(self, value):
        self._db = value

    @name.setter
    def name(self, value):
        self._name = value

    @port.setter
    def port(self, value):
        self._port = value

    @host.setter
    def host(self, value):
        self._host = value

    @user.setter
    def user(self, value):
        self._user = value

    @pw.setter
    def pw(self, value):
        self._pw = value

    @schema.setter
    def schema(self, value):
        self._schema = value







import asyncio
import pandas as pd
from sqlalchemy.inspection import inspect
from sqlalchemy import text
from .sqldict import CDMSQL
# https://gist.github.com/dermatologist/f436cb461a3290732a27c4dc040229f9
# Thank you! https://gist.github.com/garaud
class CdmVector(object):

    def __init__(self, result=None):
        self._result = result
        self._df = None

    @property
    def df(self):
        if self._df is None:
            self.create_df()
        return self._df

    @property
    def result(self):
        return self._result

    @result.setter
    def result(self, value):
        self._result = value

    def query_to_list(self):
        """List of result
        Return: columns name, list of result
        """
        result_list = []
        instance = None
        if self._result is None:
            return None, []
        for obj in self._result:
            instance = inspect(obj)
            items = instance.attrs.items()
            result_list.append([x.value for _,x in items])
        if instance is None:
            return None, []
        return instance.attrs.keys(), result_list

    def create_df(self, _names=None):
        names, data = self.query_to_list()
        if(_names):
            names = _names
        self._df = pd.DataFrame.from_records(data, columns=names)

    async def sql_df(self, cdm, sqldict=None, query=None, chunksize=1000):
        if sqldict:
            query=CDMSQL[sqldict]
        async with cdm.session() as session:
            result = await session.execute(text(query))
        self._result = result
        await session.close()
        return result



import asyncio
import pytest

@staticmethod
def test_create_vector(pyomop_fixture, metadata_fixture, vector_fixture, capsys):
    engine = pyomop_fixture.engine
    # create tables
    asyncio.run(pyomop_fixture.init_models(metadata_fixture))
    asyncio.run(create_vector(pyomop_fixture, vector_fixture, engine))


async def create_vector(pyomop_fixture, vector_fixture, engine):
    from src.pyomop import Cohort
    import datetime
    from sqlalchemy.future import select

    # Add a cohort
    async with pyomop_fixture.session() as session:
        async with session.begin():
            session.add(Cohort(cohort_definition_id=2, subject_id=100,
                cohort_end_date=datetime.datetime.now(),
                cohort_start_date=datetime.datetime.now()))
        await session.commit()

    # Query the cohort
    stmt = select(Cohort).where(Cohort.subject_id == 100)
    result = await session.execute(stmt)

    vector_fixture.result = result.scalars()
    print(vector_fixture.df.dtypes)
    assert vector_fixture.df.empty is False

    result2 = await vector_fixture.sql_df(pyomop_fixture, 'TEST')
    for row in result2:
        print(row)
    assert row.subject_id == 100

    await session.close()
    await engine.dispose()






import pandas as pd
from .cdm6_tables import Concept
import asyncio
from contextlib import asynccontextmanager
from typing import AsyncGenerator
from sqlalchemy.ext.asyncio import (
    AsyncSession,
    async_scoped_session,
)
from sqlalchemy.orm import sessionmaker
from sqlalchemy import insert
import numpy as np
from sqlalchemy.ext.automap import automap_base, AutomapBase
from sqlalchemy import select
class CdmVocabulary(object):
    def __init__(self, cdm):
        self._concept_id = 0
        self._concept_name = ''
        self._domain_id = ''
        self._vocabulary_id = ''
        self._concept_class_id = ''
        self._concept_code = ''
        self._cdm = cdm
        self._engine = cdm.engine
        self._maker = sessionmaker(self._engine, class_=AsyncSession)
        self._scope = async_scoped_session(self._maker, scopefunc=asyncio.current_task)

    @property
    def concept_id(self):
        return self._concept_id

    @property
    def concept_code(self):
        return self._concept_code

    @property
    def concept_name(self):
        return self._concept_name

    @property
    def vocabulary_id(self):
        return self._vocabulary_id

    @property
    def domain_id(self):
        return self._domain_id

    @concept_id.setter
    def concept_id(self, concept_id):
        self._concept_id = concept_id
        _concept = asyncio.run(self.get_concept(concept_id))
        self._concept_name = _concept.concept_name
        self._domain_id = _concept.domain_id
        self._vocabulary_id = _concept.vocabulary_id
        self._concept_class_id = _concept.concept_class_id
        self._concept_code = _concept.concept_code

    async def get_concept(self, concept_id):
        stmt = select(Concept).where(Concept.concept_id == concept_id)
        async with self._cdm.session() as session:
            _concept = await session.execute(stmt)
        return _concept.scalar_one()

    async def get_concept_by_code(self, concept_code, vocabulary_id):
        stmt = select(Concept).where(Concept.concept_code == concept_code) \
            .where(Concept.vocabulary_id == vocabulary_id)
        async with self._cdm.session() as session:
            _concept = await session.execute(stmt)
        return _concept.scalar_one()

    def set_concept(self, concept_code, vocabulary_id=None):
        self._concept_code = concept_code
        try:
            if vocabulary_id is not None:
                self._vocabulary_id = vocabulary_id
                _concept = asyncio.run(self.get_concept_by_code(concept_code, vocabulary_id))
            else:
                _concept = asyncio.run(self.get_concept_by_code(concept_code))
                self._vocabulary_id = _concept.vocabulary_id

            self._concept_name = _concept.concept_name
            self._domain_id = _concept.domain_id
            self._concept_id = _concept.concept_id
            self._concept_class_id = _concept.concept_class_id
            self._concept_code = _concept.concept_code

        except:
            self._vocabulary_id = 0
            self._concept_id = 0

    def create_vocab(self, folder, sample=None):
        try:
            df = pd.read_csv(folder + '/DRUG_STRENGTH.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'drug_strength', 'replace'))
            # df.to_sql('drug_strength', con=self._engine, if_exists = 'replace')
            df = pd.read_csv(folder + '/CONCEPT.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'concept', 'replace'))
            # df.to_sql('concept', con=self._engine, if_exists = 'replace')
            df = pd.read_csv(folder + '/CONCEPT_RELATIONSHIP.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'concept_relationship', 'replace'))
            # df.to_sql('concept_relationship', con=self._engine, if_exists = 'replace')
            df = pd.read_csv(folder + '/CONCEPT_ANCESTOR.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'concept_ancestor', 'replace'))
            # df.to_sql('concept_ancester', con=self._engine, if_exists = 'replace')
            df = pd.read_csv(folder + '/CONCEPT_SYNONYM.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'concept_synonym', 'replace'))
            # df.to_sql('concept_synonym', con=self._engine, if_exists = 'replace')
            df = pd.read_csv(folder + '/VOCABULARY.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'vocabulary', 'replace'))
            # df.to_sql('vocabulary', con=self._engine, if_exists = 'replace')
            df = pd.read_csv(folder + '/RELATIONSHIP.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'relationship', 'replace'))
            # df.to_sql('relationship', con=self._engine, if_exists = 'replace')
            df = pd.read_csv(folder + '/CONCEPT_CLASS.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'concept_class', 'replace'))
            # df.to_sql('concept_class', con=self._engine, if_exists = 'replace')
            df = pd.read_csv(folder + '/DOMAIN.csv', sep='\t', nrows=sample, on_bad_lines='skip')
            asyncio.run(self.write_vocab(df, 'domain', 'replace'))
            # df.to_sql('domain', con=self._engine, if_exists = 'replace')
        except Exception as e:
            print(f"An error occurred while creating the vocabulary: {e}")


    @asynccontextmanager
    async def get_session(self) -> AsyncGenerator[AsyncSession, None]:
        async with self._scope() as session:
            yield session

    async def write_vocab(self, df, table, if_exists='replace', chunk_size=1000):
        async with self.get_session() as session:
            conn = await session.connection()
            automap: AutomapBase = automap_base()
            await conn.run_sync(lambda sync_conn: automap.prepare(autoload_with=sync_conn))
            mapper = getattr(automap.classes, table)
            stmt = insert(mapper)

            for _, group in df.groupby(np.arange(df.shape[0], dtype=int) // chunk_size):
                await session.execute(stmt, group.to_dict("records"))
            await session.commit()
            await session.close()


import asyncio
import pytest

@staticmethod
def test_create_tables(pyomop_fixture, metadata_fixture, capsys):
    engine = pyomop_fixture.engine
    # create tables
    asyncio.run(pyomop_fixture.init_models(metadata_fixture))

def test_create_vocab(pyomop_fixture, metadata_fixture, capsys):
    engine = pyomop_fixture.engine
    create_vocab(pyomop_fixture, engine)

def create_vocab(pyomop_fixture,engine):
    from src.pyomop import CdmVocabulary
    vocab = CdmVocabulary(pyomop_fixture)
    vocab.create_vocab('tests', 10)
    print("Done")







# import pytest
# import os

# @pytest.fixture
# def pyomop_fixture():
#     from src.pyomop import CdmEngineFactory
#     cdm = CdmEngineFactory()
#     return cdm

# @pytest.fixture
# def metadata_fixture():
#     from src.pyomop import metadata
#     return metadata

# def test_create_vocab(pyomop_fixture, metadata_fixture, capsys):
#     from src.pyomop import CdmVocabulary
#     vocab = CdmVocabulary(pyomop_fixture)
#     vocab.create_vocab('tests', 10)
#     print("Done")



from pyomop import CdmEngineFactory, CdmVocabulary, CdmVector, Cohort, Vocabulary, metadata
from sqlalchemy.future import select
import datetime
import asyncio

async def main():
    cdm = CdmEngineFactory()  # Creates SQLite database by default
    # Postgres example (db='mysql' also supported)
    # cdm = CdmEngineFactory(db='pgsql', host='', port=5432,
    #                       user='', pw='',
    #                       name='', schema='cdm6')

    engine = cdm.engine
    # Create Tables if required
    await cdm.init_models(metadata)
    # Create vocabulary if required
    vocab = CdmVocabulary(cdm)
    # vocab.create_vocab('/path/to/csv/files')  # Uncomment to load vocabulary csv files

    # Add a cohort
    async with cdm.session() as session:
        async with session.begin():
            session.add(Cohort(cohort_definition_id=2, subject_id=100,
                cohort_end_date=datetime.datetime.now(),
                cohort_start_date=datetime.datetime.now()))
        await session.commit()

    # Query the cohort
    stmt = select(Cohort).where(Cohort.subject_id == 100)
    result = await session.execute(stmt)
    for row in result.scalars():
        print(row)
        assert row.subject_id == 100

    # Query the cohort pattern 2
    cohort = await session.get(Cohort, 1)
    print(cohort)
    assert cohort.subject_id == 100

    # Convert result to a pandas dataframe
    vec = CdmVector()
    vec.result = result
    print(vec.df.dtypes)

    result = await vec.sql_df(cdm, 'TEST') # TEST is defined in sqldict.py
    for row in result:
        print(row)

    result = await vec.sql_df(cdm, query='SELECT * from cohort')
    for row in result:
        print(row)


    # Close session
    await session.close()
    await engine.dispose()

# Run the main function
asyncio.run(main())


import sys

from .engine_factory import CdmEngineFactory
from .cdm6_tables import metadata
from .vocabulary import CdmVocabulary
from .vector import CdmVector

from .cdm6_tables import AttributeDefinition
from .cdm6_tables import CareSite
from .cdm6_tables import CdmSource
from .cdm6_tables import Cohort
from .cdm6_tables import CohortAttribute
from .cdm6_tables import CohortDefinition
from .cdm6_tables import Concept
from .cdm6_tables import ConceptAncestor
from .cdm6_tables import ConceptClass
from .cdm6_tables import ConceptRelationship
from .cdm6_tables import ConceptSynonym
from .cdm6_tables import ConditionEra
from .cdm6_tables import ConditionOccurrence
from .cdm6_tables import Death
from .cdm6_tables import DeviceCost

from .cdm6_tables import DeviceExposure
from .cdm6_tables import Domain
from .cdm6_tables import DoseEra
from .cdm6_tables import DrugCost
from .cdm6_tables import DrugEra

from .cdm6_tables import DrugExposure
from .cdm6_tables import DrugStrength
from .cdm6_tables import FactRelationship
from .cdm6_tables import Location
from .cdm6_tables import Measurement

from .cdm6_tables import Note
from .cdm6_tables import Observation
from .cdm6_tables import ObservationPeriod
from .cdm6_tables import PayerPlanPeriod
from .cdm6_tables import Person

from .cdm6_tables import ProcedureCost
from .cdm6_tables import ProcedureOccurrence
from .cdm6_tables import Provider
from .cdm6_tables import Relationship
from .cdm6_tables import SourceToConceptMap

from .cdm6_tables import Speciman
from .cdm6_tables import VisitCost
from .cdm6_tables import VisitOccurrence
from .cdm6_tables import Vocabulary

if sys.version_info[:2] >= (3, 8):
    # TODO: Import directly (no need for conditional) when `python_requires = >= 3.8`
    from importlib.metadata import PackageNotFoundError, version  # pragma: no cover
else:
    from importlib_metadata import PackageNotFoundError, version  # pragma: no cover

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = __name__
    __version__ = version(dist_name)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"
finally:
    del version, PackageNotFoundError



import asyncio
import pytest

@staticmethod
def test_create_patient(pyomop_fixture, metadata_fixture, capsys):
    engine = pyomop_fixture.engine
    # create tables
    asyncio.run(pyomop_fixture.init_models(metadata_fixture))
    asyncio.run(create_patient(pyomop_fixture, engine))


async def create_patient(pyomop_fixture,engine):
    from src.pyomop import Person
    from src.pyomop import Cohort
    import datetime
    from sqlalchemy.future import select

    # Add a patient
    async with pyomop_fixture.session() as session:
        async with session.begin():
            session.add(Person(
                gender_concept_id=100,
                year_of_birth=2000,
                race_concept_id=200,
                ethnicity_concept_id=300
            ))
        await session.commit()

    # Add couple of cohorts
    async with pyomop_fixture.session() as session:
        async with session.begin():
            session.add(Cohort(cohort_definition_id=2, subject_id=1,
                cohort_end_date=datetime.datetime.now(),
                cohort_start_date=datetime.datetime.now()))
            session.add(Cohort(cohort_definition_id=3, subject_id=1,
                cohort_end_date=datetime.datetime.now(),
                cohort_start_date=datetime.datetime.now()))
        await session.commit()

    # Query the cohort for patient 1
    stmt = select(Cohort).where(Cohort.subject_id == 1).order_by(Cohort._id)
    result = await session.execute(stmt)
    for row in result.scalars():
        print(row)
        assert row.subject_id == 1
